/*
 * Copyright (c) 2001-2003 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#include "lwip/ip_addr.h"

#define Eth_INET
#include "eth_lwip.h"
//#include "lwip/inet.h"


/* used by IP_ADDR_ANY and IP_ADDR_BROADCAST in ip_addr.h */
const struct ip_addr ip_addr_any = { 0x00000000UL };
const struct ip_addr ip_addr_broadcast = { 0xffffffffUL };


void IP4_ADDR(struct ip_addr* ipaddr, u32_t a,u32_t b,u32_t c,u32_t d)
{
 (ipaddr)->addr = htonl(((u32_t)(a & 0xff) << 24) | ((u32_t)(b & 0xff) << 16) | \
                ((u32_t)(c & 0xff) << 8) | (u32_t)(d & 0xff));
}

void ip_addr_set(struct ip_addr* dest, struct ip_addr* src)
{	 
	(dest)->addr = \
        ((src) == NULL? 0:\
         ((struct ip_addr *)src)->addr);
}
u8_t ip_addr_maskcmp(struct ip_addr* addr1,struct ip_addr* addr2,struct ip_addr* mask)
{
	 return(((addr1)->addr & \
      (mask)->addr) == \
      ((addr2)->addr & \
       (mask)->addr));
}
u8_t ip_addr_cmp(struct ip_addr* addr1,struct ip_addr* addr2)
{
	 return((addr1)->addr == (addr2)->addr);
}

u8_t ip_addr_isany(struct ip_addr* addr1)
{
	return ((addr1) == NULL || (addr1)->addr == 0);
}

u8_t ip_addr_isbroadcast(struct ip_addr* addr1,struct ip_addr* mask)
{ 
   return	(((((addr1)->addr) & ~((mask)->addr)) == \
        (0xffffffff & ~((mask)->addr))) || \
          ((addr1)->addr == 0xffffffff) || \
               ((addr1)->addr == 0x00000000));
}

u8_t ip_addr_ismulticast(struct ip_addr* addr1)
{
 return(((addr1)->addr & ntohl(0xf0000000)) == ntohl(0xe0000000));
}
/*
#define ip_addr_debug_print(debug, ipaddr) LWIP_DEBUGF(debug, ("%u.%u.%u.%u", \
        ipaddr?(unsigned int)(ntohl((ipaddr)->addr) >> 24) & 0xff:0, \
        ipaddr?(unsigned int)(ntohl((ipaddr)->addr) >> 16) & 0xff:0, \
        ipaddr?(unsigned int)(ntohl((ipaddr)->addr) >> 8) & 0xff:0, \
        ipaddr?(unsigned int)ntohl((ipaddr)->addr) & 0xff:0U))
*/
/* cast to unsigned int, as it is used as argument to printf functions
 * which expect integer arguments */
unsigned int ip4_addr1(struct ip_addr* ipaddr)
{
	return ((unsigned int)(ntohl((ipaddr)->addr) >> 24) & 0xff);
}
unsigned int ip4_addr2(struct ip_addr* ipaddr)
{
	return ((unsigned int)(ntohl((ipaddr)->addr) >> 16) & 0xff);
}
unsigned int ip4_addr3(struct ip_addr* ipaddr)
{
 return((unsigned int)(ntohl((ipaddr)->addr) >> 8) & 0xff);
 }
unsigned int ip4_addr4(struct ip_addr* ipaddr)
{
	return ((unsigned int)(ntohl((ipaddr)->addr)) & 0xff);
}
